@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require "rbs/version"
 
 require "set"
@@ -7,9 +9,7 @@ require "pp"
 require "ripper"
 require "logger"
 require "tsort"
-require "strscan"
 
-require "rbs/char_scanner"
 require "rbs/errors"
 require "rbs/buffer"
 require "rbs/namespace"
@@ -30,6 +30,8 @@ require "rbs/definition_builder/method_builder"
 require "rbs/variance_calculator"
 require "rbs/substitution"
 require "rbs/constant"
+require "rbs/resolver/constant_resolver"
+require "rbs/resolver/type_name_resolver"
 require "rbs/constant_table"
 require "rbs/ast/comment"
 require "rbs/writer"
@@ -59,12 +61,12 @@ module RBS
     attr_reader :logger_output
 
     def logger
-      @logger ||= Logger.new(logger_output || STDERR, level: logger_level || "warn", progname: "rbs")
+      @logger ||= Logger.new(logger_output || STDERR, level: logger_level || Logger::WARN, progname: "rbs")
     end
 
     def logger_output=(val)
-      @logger_output = val
       @logger = nil
+      @logger_output = val
     end
 
     def logger_level=(level)
